

class MDRdTest(unittest.TestCase):
	"""SIA 263:13, 4.5.2 Kippen von Biegeträgern
	C4/06, 3.2 Anwendungsbeispiel zum Kippnachweis
	"""

	def setUp(self):
		self.entries = {
			'steel_grade': 'S 235',
			'shape': 'IPE',
			'size': '600',
			'psi': 0.5,
			'ld': 4000,
			'production': 'rolled',
			'w': 'wply',
			'sub_code': 'sia263e',
			
		}

	def testCalculation(self):
		mcr = MCR(self.entries)  # requires ld, psi
		self.assertEqual(round(mcr.eta(), 2), 1.30)
		self.assertEqual(round(mcr.i('double_symmetric')), 56)
		self.assertEqual(round(mcr.sigma_dw()), 528)
		self.assertEqual(round(mcr.c()), 998000)
		self.assertEqual(round(mcr.sigma_dv()), 324)
		self.assertEqual(round(mcr.sigma_crd()), 620)
		printf('MCR', mcr.get_results())
		
		mdrd = MDRd(self.entries)
		self.assertEqual(mdrd.alpha_d(), 0.21)
		self.assertEqual(round(mdrd.lambda_d_(), 3), 0.66)
		self.assertEqual(round(mdrd.chi_d(), 4), 0.9166)

		print('mdrd: {}'.format(mdrd.mdrd()))
		printf('MdRd', mdrd.get_results())

class NKRdTest(unittest.TestCase):
	"""SIA 263:13, 4.5.1 Knicken
	C4/06, 2.2 Anwendungsbeispiel
	"""
	
	def setUp(self):
		self.entries = {
			'steel_grade': 'S 275',
			'shape': 'HEA',
			'size': '180',
			'lk': 4000,  # [mm]
			'production': 'rolled',
		}

	def testCalculation(self):
		nkrd = NkRd(self.entries)
		self.assertEqual(nkrd.alpha('z'), 0.49)
		self.assertEqual(round(nkrd.lambda_k_z(), 2), 1.02)
		self.assertEqual(round(nkrd.nkrd_z()*1e-3, 0), 627)
		print('NkRd: {}'.format(nkrd.phi_k_z()))
		print('NkRd: {}'.format(nkrd.chi_k_z()))
		print('NkRd: {}'.format(nkrd.nkrd_z()))
		printf('NkRd', nkrd.get_results())


class TablesTest(unittest.TestCase):
	"""SIA 263:13, 4.5.1 Knicken
	C4/06, 2.2 Anwendungsbeispiel
	"""
	
	def setUp(self):
		self.db = Database()

	def testCalculation(self):
		profiles = self.db.select_tables()
		print(profiles)

		for p in profiles:
			header = self.db.select_header(p)
			print(header)
			table = self.db.select_table(p)

			def convert(s):
				s = str(s)
				if s.isdigit():
					if '.' in s:
						return float(s)
					else:
						return float(s)
				else:
						return s
					
			for row in table:
				#rowf = [convert(x) for x in row]


				s = ''
				i = 0
				for h in header:
					if h == 'dmax':
						s += '{'+str(i)+':>9}'
					else:
						s += '{'+str(i)+':>9}'
					i += 1

				print(s.format(*row))
					

class TablesTest2(unittest.TestCase):
	"""SIA 263:13, 4.5.1 Knicken

	"""

	def setUp(self):
		self.db = Database()
		self.entries = {
			'steel_grade': 'S 235',
			'shape': 'HEA',
			'size': '180',
			'lk': 4000,  # [mm]
			'production': 'rolled',
		}

	def testCalculation(self):
		profiles = self.db.select_tables()

		for p in profiles:
			print(p, self.entries['steel_grade'])
			self.entries['shape'] = p
			header = self.db.select_header(p)
			table = self.db.select_table(p)
			
			length = [0, 1, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5, 7, 7.5, 8, 8.5, 9, 9.5, 10]
			s = ''
			for i, _ in enumerate(length):
				s += '{'+str(i)+':>6}'
			print(s.format(*length))

			
			for i, row in enumerate(table):
				self.entries['size'] = row[0]
				row_dict = dict(zip(header, row))

				nkrd = []
				
				for l in length:
					self.entries['lk'] = l*1e3
					nkrd.append(round(NkRd(self.entries).nkrd_y()*1e-3))
				
				if i % 5 == 0:
					print('')

				
				
				print(s.format(*nkrd))
				
				
				
				


if __name__ == "__main__":
	unittest.main() 




from math import sqrt
from .params import beton_dict, betonstahl_dict, es
from ..results import Results

code = 'SIA 262:13'


def index(name):
	data = {
		
	}
	return data.get(name, '')


def kt(w, h):
	""" (99) kt [-]"""
	kt = 1/(1+0.5*t(w, h))
	return kt


def t(w, h):
	""" () t [m] """
	# jeweils kleinste Abmessung massgebend
	# term = min(b, h)
	# für Platten- und Rechteckquerschnitte
	term = h/3 * 1e-3  # t in m
	return term


class Beton(Results):
	code = code
	
	def __init__(self, choice, gamma_c, eta_t=1.0):
		key = list(beton_dict)[choice]
		self.fck, self.fctm, self.beta_fc = beton_dict[key]
		self.gamma_c = gamma_c
		self.eta_t_ = eta_t

	def fck_(self):
		print('fck from cat262: {}'.format(self.fck))
		return self.fck

	def fcd(self):
		""" (2) """
		term = self.eta_fc()*self.eta_t()*self.fck/self.gamma_c
		term = round(term*2)/2
		self.add_rslt('fcd', term, 'f.cd', 'N/mm^2', '2.3.2.3', 2)
		return term

	def eta_fc(self):
		""" (26) """
		term = (30/self.fck)**(1/3)
		term = 1.0 if term >= 1.0 else term
		self.add_rslt('eta_fc', term, '&eta;.fc', '-', '4.2.1.2', 26)
		return term

	def eta_t(self):
		""" (27) """
		if isinstance(self.eta_t_, tuple):
			fcm_tL, fcm_tP = self.eta_t_
			term = 0.85*fcm_tL/fcm_tP
			term = 1.0 if term >= 1.0 else term
		else:
			term = self.eta_t_
		self.add_rslt('eta_t', term, '&eta;.t', '-', '4.2.1.3', 27)
		return term

	def tau_cd(self):
		""" (3) """
		term = 0.3*self.eta_t()*sqrt(self.fck)/self.gamma_c
		term = round(term*2, 1)/2
		self.add_rslt('tau_cd', term, '&tau;.cd', '-', '2.3.2.4', 3)
		return term

	def fctd(self, w, h):
		""" (98) """
		term = kt(w, h)*self.fctm
		self.add_rslt('fctd', round(term, 1), 'f.ctd', 'N/mm^2', '4.4.1.3', 98)
		return term
	



class Betonstahl(Results):
	code = code
	
	def __init__(self, choice, gamma_s):
		key = list(betonstahl_dict)[choice]
		self.fsk, self.ks, self.epsilon_ud = betonstahl_dict[key]
		self.gamma_s = gamma_s

	def epsilon_yd(self):
		return self.fsd()/es

	def fsd(self):
		""" (4) fsd [N/mm2] """
		term = self.fsk/self.gamma_s
		term = round(term)
		self.add_rslt('fsd', term, 'f.sd', 'N/mm^2', '2.3.2.5', 4)
		return term

	def epsilon_v_el(self, md, mrd):
		"""" (38) """
		term = (self.fsd()/es)*(md/mrd)
		self.add_rslt('epsilon_v_el', round(term, 4), '\u03B5.v,el', '-', '4.3.3.2.2', '38')
		return term

	def epsilon_v_pl(self):
		""" (39) """
		term = 1.5*self.fsd()/es
		self.add_rslt('epsilon_v_pl', round(term, 4), '\u03B5.v,pl', '-', '4.3.3.2.2', '39')
		return term


class Cat262(Results):
	code = code

	def __init__(self):
		pass

	# 4.3.3 Shear Force
	
	# 4.3.3.2 Shear Force without Reinforcement

	def vrd(self, kd, tau_cd, d_v):
		""" (35) """
		term = kd*tau_cd*d_v
		self.add_rslt('vrd', round(term, 1), 'v.Rd', 'N/mm', '4.3.3.2.1', 35)
		return term

	def kd(self, epsilon_v, d, kg):
		""" (36) """
		term = 1/(1+epsilon_v*d*kg)
		self.add_rslt('kd', round(term, 2), 'k.d', '-', '4.3.3.2.1', 36)
		return term

	def kg(self, dmax):
		""" (37) """
		term = 48/(16+dmax)
		self.add_rslt('kg', round(term, 2), 'k.g', '-', '4.4.4.2.1', 37)
		return term


	




import unittest
from civeng.steel.props import Square

class SquareTest(unittest.TestCase):
	
	def setUp(self):
		self.entries = (40, 40, 3)
	
	def testCalculation(self):
		rrk = Square(*self.entries)

		print(rrk.m())
		print(rrk.a())
		print(rrk.i())
		print(rrk.wel())


if __name__ == "__main__":
	unittest.main() 



from .params import rho
from math import pi

class Square:

	def __init__(self, h, b, t):
		self.h = h
		self.b = b
		self.t = t

	def m(self):
		return self.a()*1e-6*rho

	def a(self):
		return self.t**2*(4*(self.h/self.t-4)+3*pi)

	def i(self):
		i_h = self.t*(self.h-4*self.t)**3/12 * 2
		i_b = (self.b-4*self.t)*self.t**3/12 * 2
		i_t = (pi/8-8/(9*pi))*15*self.t**4 * 2
		az_b = (self.b-4*self.t)*self.t * ((self.h-self.t)/2)**2 * 2
		az_t = pi/2*3*self.t**2 * ((self.h/2-2*self.t)+self.t)**2 * 2
		return i_h+i_b+i_t+az_b+az_t
	
	def wel(self):
		return self.i()/(self.h/2)




import inspect

def descr(name):
	
	descr = {
		'as_max_stat_d': 'Max. Bewehrung für stat. best. Sys.',
		'as_max_stat_ind': 'Max. Bewehrung für stat. unbest. Sys.',
		'as_min': 'Mindestbewehrung',
		'epsilon_s_inf': 'Stahldehnung unten',
		'epsilon_v_el': 'Dehnung im elastischen Zustand',
		'eta_fc': 'Umrechnungsfaktor zur Berücksichtigung des Bruchverhaltens',
		'eta_t': 'Umrechnungsfaktor für Betonfestigkeiten',
		'fcd': 'Bemessungswert der Betondruckfestigkeit',
		'fctd': 'Bemessungswert der Betonzugfestigkeit',
		'fsd': 'Bemessungswert der Fliessgrenze Betonstahl',
		'kd': 'Beiwert zur Bestimmung des Querkraftwiderstands',
		'kg': 'Beiwert zur Berücksichtigung des Grösstkorns',
		'mrd': 'Widerstandsmoment',
		'mrd_': 'Rissmoment',
		'mrd_max_stat_d': 'Max. Widerstandsm. für stat. best. Sys.',
		'mrd_max_stat_ind': 'Max. Widerstandsm. für stat. unbest. Sys.',
		'tau_cd' : 'Bemessungswert der Schubspannungsgrenze',
		'vrd': 'Bemessungswert des Querkraftwiderstands',
		'x': 'Druckzonenhöhe',
		'z': 'Innerer Hebelarm',
	}

	return descr.get(name, '')


def descr2(name):
	
	descr = {
		'epsilon_s_inf': ('', '', '\u03B5.s,inf', '\u2030', 'Stahldehnung unten'),
	}
	return descr.get(name, '')



class Results:
	resultsf = {}
	messages = {}

	def __init__(self):
		pass

	def add_rslt(self, name, rslt, lbl, unit, ff='', f=''):
		
		self.resultsf[name] = {
			'name': name,
			'standard': self.code,
			'ff': '{} {}'.format(self.code, ff),
			'f': f,
			'description': descr(name),
			'label': lbl,
			'result': rslt,
			'unit': unit,
		}


	def add_rslt2(self, result, reference=''):
		name = inspect.stack()[1][3]
		ff, f, label, unit, description = descr2(name)
		self.resultsf[name] = {
			'name': name,
			'standard': self.code,
			'ff': '{} {}'.format(self.code, ff),
			'f': f,
			'description': description,
			'label': label,
			'result': result,
			'unit': unit,
		}
		if reference and result > reference:
			comparison = '{} > {}'.format(result, reference)
			self.resultsf[name].update({'result': comparison})
			self.messages[name] = {'name': name}


	def add_msg(self, name, msg):
		self.messages[name] = msg
		

	def sorted_results(self):
		return [self.resultsf[k] for k in sorted(self.resultsf)]

	def quick_results(self):
		return [self.resultsf[k] for k in self.selection]
	
	def get_results(self):
		print('messages: {}'.format(self.messages))
		results = {'main_rslts': self.sorted_results(), 
				'quick_rslts': self.quick_results(),
				'messages': self.messages.copy()}
		self.resultsf.clear()
		self.messages.clear()
		return results
	







from ..cat263 import Components, MDRd, NkRd
from ...results import Results


class Amazius(Results):
	code = 'SIA 263:13'
	selection = ['nrd', 'vrd', 'mrdy', 'mrdz', 'mdrd', 'nkrd_y', 'nkrd_z']

	def __init__(self, entries):

		self.cat = Components(entries)
		self.cross_section_class = entries['cross_section_class']
		self.ned = entries['ned']
		self.ved = entries['ved']
		self.myed = entries['medy']
		self.mzed = entries['medz']
		self.entries = entries

	def add_sel(self, sel):
		if sel not in self.selection:
			self.selection.append(sel)
		

	def compute(self):
		mdrd = MDRd(self.entries)
		mdrd.mdrd()
		nkrd = NkRd(self.entries)
		nkrd.nkrd_y()
		nkrd.nkrd_z()
		
		if self.cross_section_class == '1 + 2':
			self.class_1()

		return  self.get_results()

	def class_1(self):
		self.cat.nrd()
		self.cat.myrd()
		self.cat.mzrd()
		self.cat.vrd()

		if self.ved > 0:
			self.cat.mvrd(self.ved)
			self.add_sel('mvrd')
						
		if self.ned > 0 and self.myed > 0:
			self.cat.f49(self.ned, self.mzed, 'y')
			self.add_sel('f49_y')
		
		if self.ned > 0 and self.mzed > 0:
			self.cat.f49(self.ned, self.mzed, 'z')
			self.add_sel('f49_z')

		if self.ned > 0 and self.myed > 0 and self.mzed > 0:
			self.cat.f44(self.ned, self.myed, self.mzed)
			self.add_sel('f44')

			self.cat.f48(self.ned, self.myed, self.mzed)
			self.add_sel('f48')

			self.cat.f50(self.ned, self.myed, self.mzed)
			self.add_sel('f50')
		
		if self.ned > 0:
			self.cat.mynrd(self.ned)
			self.add_sel('mynrd')

			self.cat.mznrd(self.ned)
			self.add_sel('mznrd')

	def vrd(self):
		pass



 




from .params import steel_grade_dict
from .szs import Database
from .params import gamma_m1, gamma_m2, e, g
from ..results import Results
from math import pi, sqrt

code = 'SIA 262:13'




class Profile():
	code = 'SIA 262:13'
	selection = ''

	def __init__(self, entries):
		steel_grade = entries['steel_grade']
		self.shape = entries['shape']
		size = entries['size']
		
		self.gamma_m1 = gamma_m1
		self.gamma_m2 = gamma_m2

		db = Database()
		data = db.select_row_dict(self.shape, size)

		self.a = float(data['a'])
		self.av = float(data['av'])
		self.aw = float(data['aw'])
		self.iy = float(data['iy'])*1e6
		self.wely = float(data['wely'])*1e3
		self.wy = float(data['wy'])*1e3
		self.wply = float(data['wply'])*1e3
		self.iy_ = float(data['iy_'])
		self.iz = float(data['iz'])*1e6
		self.welz = float(data['welz'])*1e3
		self.wplz = float(data['wplz'])*1e3
		self.iz_ = float(data['iz_'])
		self.k = float(data['k'])*1e6
		
		self.h = float(data['h'])
		self.b = float(data['b'])
		self.tw = float(data['tw'])
		self.tf = float(data['tf'])
		self.r = float(data['r'])

		self.h1 = float(data['h1'])
		self.k_ = float(data['k_'])
		self.a_ = float(data['a_'])
		self.h2 = float(data['h2'])
		#self.w = float(data['w'])
		#self.dmax = float(data['dmax'])

		t = max(self.tf, self.tw)
		if t <= 40:
			self.fy, self.tau_y, self.fu = steel_grade_dict[steel_grade][0]
		elif t <= 100:
			self.fy, self.tau_y, self.fu = steel_grade_dict[steel_grade][1]
			
		self.profiles_I = [
			'IPE', 'PEA', 'INP', 'HEA', 'HEB', 'HEM', 'HHD', 'HL']
		self.profiles_R = ['RRK', 'RRW']



class Profile2(Profile):
	
	def __init__(self, entries):
		Profile.__init__(self, entries)

		
		steel_grade2 = entries['steel_grade2']
		self.shape2 = entries['shape2']
		size2 = entries['size2']
		
	

class Components(Profile, Results):
	code = code
	selection = ''

	def __init__(self, entries):
		Profile.__init__(self, entries)
		self.entries = entries
		if 'lk' in entries:
			self.lk = entries['lk']
		else:
			self.lk = entries['ld']

		self.ld = self.lk
		self.psi_y = entries['psi']  # !!!!!!!!!!!!!!!!!!!!
		self.psi_z = entries['psi']  #!!!!!!!!!!!!!!!!!!!!!!

	def nrd(self):
		""" (38) """
		term = self.fy*self.a/self.gamma_m1
		self.add_rslt('nrd', round(term*1e-3), 'N.Rd', 'kN', '5.1.2.1', 38)
		return term

	def myrd(self):
		""" (40) """
		term = self.fy*self.wply/self.gamma_m1
		self.add_rslt('mrdy', round(term*1e-6, 1), 'M.y.Rd', 'kNm', '5.1.3.1', 40)
		return term

	def mzrd(self):
		""" (40) """
		term = self.fy*self.wplz/self.gamma_m1
		self.add_rslt('mrdz', round(term*1e-6, 1), 'M.z.Rd', 'kNm', '5.1.3.1', 40)
		return term

	def vrd(self):
		""" (41) """
		term = self.tau_y*self.av/self.gamma_m1
		self.add_rslt('vrd', round(term*1e-3, 1), 'V.Rd', 'kN', '5.1.4.1', 41)
		return term

	def f42(self):
		"""(42)"""
		term = (self.h-self.tf)/self.tw/sqrt((4*e)/self.fy)
		self.add_rslt('f42', round(term, 3), 'f42', '5.1.4.2', 42)
		return term

	def av__(self):
		"""(42a)"""
		term = self.a-2*self.b*self.tf+(self.tw+2*self.r)*self.tf
		self.add_rslt('av', round(term, 1), 'A.v', '5.1.4.3', 43)
		return term

	def mvrd(self, ved):
		""" (43) """
		if self.shape in self.profiles_I:
			t1, t2, hw = self.tf, self.tw, self.h2
		else:
			t = self.t
			t1, t2, hw = t, 2*t, self.h-3*t

		term = self.b*t1*self.fy*(self.h-t1)/self.gamma_m1+hw**2*t2*self.fy/ \
			(4*self.gamma_m1)*(1-(ved*1e3/self.vrd())**2)
		self.add_rslt('mvrd', round(term*1e-6, 1), 'M.\u03BD.Rd', 'kNm', '5.1.5.2', 43)
		return term

	def f44(self, ned, myed, mzed):
		""" (44) """
		term = ned*1e3/self.nrd()+ myed*1e6/self.myrd()+ mzed*1e6/self.mzrd()
		self.add_rslt('f44', round(term, 3), 'f.44', '-', '5.1.6.1', 44)
		return term

	def mynrd(self, ned):
		"""(45)"""
		if self.shape in self.profiles_I:
			a = self.a__(self.b, self.tf)
		else:
			a = self.a__(self.b, self.t)
		zeta = self.zeta_(a, 'y')
		term = self.myrd()*zeta*(1-self.n(ned))
		if term >= self.myrd():
			term = self.myrd()
		self.add_rslt('mynrd', round(term*1e-6, 1), 'M.y,N,Rd', 'kNm', '5.1.6.2', 45)
		return term

	def mznrd(self, ned):
		if self.shape in self.profiles_I:
			a = self.a__(self.b, self.tf)
			zeta = self.zeta_(a, 'z')
			if self.n(ned) > a:
				""" (46) """
				term = self.mzrd()*(1-((self.n(ned)-a)/(1-a))**2)
				f = 46
			else:
				""" (47) """
				term = self.mzrd()
				f = 47
		else:
			a = self.a__(self.h, self.t)
			zeta = self.zeta_(a, 'z')
			term = term = self.mrdy()*self.zeta*(1-self.n(ned*1e3))
			if term >= self.mrdy():
				term = self.mrdy()
		self.add_rslt('mznrd', round(term*1e-6, 1), 'M.z,N,Rd', 'kNm', '5.1.6.2', f)
		return term

	def n(self, ned):
		term = ned*1e3/self.nrd()
		return term

	def a__(self, bh, t):
		term = (self.a-2*bh*t)/self.a
		return term

	def zeta_(self, a, axis):
		term = 1/(1-0.5*a)
		self.add_rslt('zeta_'+axis, round(term, 3), '\u03B6.'+axis, '-', '5.1.6.2')
		return term

	def f48(self, ned, myed, mzed):
		if self.shape in self.profiles_I:
			alpha = 2
			beta = 5*ned*1e3/self.nrd()
			if beta <= 1.1:
				beta = 1.1
		else:
			alpha = 1.66/(1-1.13*(ned*1e3/self.nrd())**2)
			beta = alpha
			if alpha >= 6.0:
				alpha = 6.0
				beta = alpha
		term = (myed*1e6/self.mynrd(ned))**alpha+(mzed*1e6/self.mznrd(ned))**beta
		self.add_rslt('f48', round(term, 3), 'f48', '-', '5.1.6.4', 48)
		return term

	def f49(self, ned, myed, axis):
		
		def zeta_(a, axis):
			term = 1/(1-0.5*a)
			if term >= 1+0.2*self.n(ned):
				term = 1+0.2*self.n(ned)
			return term

		if self.shape in self.profiles_I:
			if axis == 'y':
				a = self.a__(self.b, self.tf)
				zeta = zeta_(a, 'y')
			else:
				zeta = 1.0
		elif self.shape in self.profiles_R:
			if axis == 'y':
				a = self.a__(self.b, self.t)
				zeta = zeta_(a, 'y')
			else:
				a = self.a__(self.h, self.t)
				zeta = zeta_(a, 'z')
		else:
			zeta = 1.0
		self.add_rslt('zeta_'+axis, round(zeta, 3), '\u03B6.'+axis, '-', '5.1.6.2')

		ncr = e*getattr(self, 'i'+axis)*pi/self.lk**2
		omega = 0.6+0.4*getattr(self, 'psi_'+axis)
		if omega <= 0.4:
			omega = 0.4
		nkrd = getattr(NkRd(self.entries), 'nkrd_'+axis)()
		print('nkrd: {}'.format(nkrd))
		term = ned*1e3/nkrd + \
			   (1/(1-ned*1e3/ncr))*(omega*myed*1e6/(self.myrd()*zeta))
		self.add_rslt('f49_'+axis, round(term, 3), 'f49.'+axis, '-', '5.1.9.1', 49)
		return term
		
	def f50(self, ned, myed, mzed):
		
		def omega(psi, axis):
			term = 0.6+0.4*getattr(self, 'psi_'+axis)
			if term <= 0.4:
				term = 0.4
			return term

		def ncr(axis):
			term = e*getattr(self, 'i'+axis)*pi/self.lk**2
			return term

		nkrd_ = NkRd(self.entries)
		nkrd = min(nkrd_.nkrd_y(), nkrd_.nkrd_z())

		term = ned*1e3/nkrd + \
			   omega(self.psi_y, 'y')/(1-ned*1e3/ncr('y'))*myed*1e6/MDRd(self.entries).mdrd() +\
			   omega(self.psi_z, 'z')/(1-ned*1e3/ncr('z'))*mzed*1e6/self.mzrd()
		self.add_rslt('f50', round(term, 3), 'f50', '-', '5.1.10.1', 50)
		return term
		

class MCR(Profile, Results):
	"""SIA 263:13, Anhang B: Ideelles Kippmoment MCR 

	:param geometry: Wheter the profile ist double_symmetric or general
	:param ld: Kipplänge
	:param psi: Verhältnis des kleineren Endmoments zum grösseren

	"""
	selection = ''

	def __init__(self, entries):
		Profile.__init__(self, entries)
		if 'ld' in entries:
			self.ld = entries['ld']
		else:
			self.ld = entries['lk']
		self.psi_y = entries['psi']  # !!!!!!!!!!!!!!!!!!!!!!!

	def eta(self):
		"""B.6 Berücksichtigt Lagerung und Biegebeanspruchung"""
		term = 1.75-1.05*self.psi_y+0.3*self.psi_y**2
		if term <= -0.5:
			term = -0.5
		self.add_rslt('eta', round(term, 2), '\u03C8', '-', 'B.6', 95)
		return term

	def lk(self):
		"""B.5 Reduzierte Kipplänge"""
		term = self.ld/sqrt(self.eta())
		self.add_rslt('lk', round(term, 1), 'L.K', 'mm', 'B.5', 94)
		return term

	def i(self, geometry = 'double_symmetric'):
		"""B.5 Trägheitsradius des Druckgurtes"""

		if geometry == 'double_symmetric':
			term = sqrt((self.iz/2)/(self.b*self.tf+self.aw/6))
		else:
			hc = (self.h-self.tf)/2
			izfc = self.b**3*self.tf/12
			izwc = self.tw**3*(hc/3-self.tf/2)/12
			afc = self.b*self.tf
			awc = self.tw*(hc/3-self.tf/2)
			term = sqrt((izfc+izwc)/(afc+awc))

		term = round(term)
		self.add_rslt('id', term, 'i.D', 'mm', 'B.5', 94)
		return term

	def lambda_k(self):
		"""B.5 Schlankheit des Druckgurtes"""
		term = self.lk()/self.i()
		self.add_rslt('lambda_k', round(term), '\u03BB', '-', 'B.5', 94)
		return term

	def sigma_dw(self):
		"""B.5 Wölbanteil sigma_dw der ideellen Kippspannung"""
		# Ziffer 5.6.2.3 ist nicht berücksichtigt
		term = pi**2*e/self.lambda_k()**2
		self.add_rslt('sigma_dw', round(term, 1), '\u03A3.Dw', 'N/mm^2', 'B.5', 94)
		return term

	def c(self):
		"""B.4 Hilfswert C"""
		term = pi/(self.wely)*sqrt(g*self.k*e*self.iz)
		term = round(term*1e-3)*1e3
		self.add_rslt('c', term*1e-3, 'C', 'kN/mm', 'B.4', 93)
		return term

	def sigma_dv(self):
		"""B.4 Saint-Venantsche Anteil sigma_dv der ideellen Kippspannung"""
		# Ziffer 5.6.2.3 ist nicht berücksichtigt
		term = self.eta()*self.c()/self.ld
		self.add_rslt('sigma_dv', round(term, 1), '\u03A3.Dv', 'N/mm^2', 'B.4', 93)
		return term

	def sigma_crd(self):
		"""B.3 Ideelle Kippspannung"""
		term = sqrt(self.sigma_dv()**2+self.sigma_dw()**2)
		self.add_rslt('sigma_crd', round(term, 1), '\u03A3.cr,D', 'N/mm^2', 'B.3', 92)
		return term

	def mcr(self):
		"""B.1 Ideelles Kippmoment"""
		term = self.wely*self.sigma_crd()
		self.add_rslt('mcr', round(term, 1), 'M.cr', 'kNm', 'B.1', 91)
		return term


class MDRd(Profile, Results):
	""" SIA 213:13, 4.5.2 Kippen von Biegeträgern 

	:param production: Wheter the profile is rolled or welded
	
	"""

	def __init__(self, entries):
		Profile.__init__(self, entries)
		self.production = entries['production']
		self.sigma_crd = MCR(entries).sigma_crd()
		self.w = getattr(self, entries['w'])
		self.sub_code = entries['sub_code']
		self.entries = entries

	def alpha_d(self):
		"""4.5.2.3 Imperfektionsbeiwerte"""
		if self.production == 'rolled':
			term = 0.21
		else:
			term = 0.49
		self.add_rslt('alpha_d', term, '\u03B1.D', '-', '4.5.2.3')
		return term

	def lambda_d_(self):
		"""4.5.2.3 Kippschlankheit"""
		term = sqrt(self.fy/round(self.sigma_crd)*(round(self.w/self.wely, 2)))
		self.add_rslt('lambda_d_', round(term, 4), '\u03BB.D', '-', '4.5.2.3')
		return term
	
	def phi_d(self):
		"""4.5.2.3"""
		lambda_d_ = self.lambda_d_()
		term = 0.5*(1+self.alpha_d()*(lambda_d_-0.4)+lambda_d_**2)
		self.add_rslt('chi_d', round(term, 1), '\u03D5.D', '-', '4.5.2.3')
		return term

	def chi_d(self):
		"""4.5.2.3 Abminderungsfaktor"""
		phi_d = self.phi_d()
		term = 1/(phi_d+sqrt(phi_d**2-self.lambda_d_()**2))
		if term >= 1.0:
			term = 1.0
		self.add_rslt('chi_d', round(term, 4), '\u03C7.D', '-', '4.5.2.3', 10)
		return term

	def mdrd(self):
		"""4.5.2.2 Kippwiderstand MD,Rd """
		if self.sub_code == 'sia263':
			chi = self.chi_k()
		else:
			chi = self.chi_d()
		print('chi: {}, {}'.format(chi, self.lambda_d_()))
		term = chi*self.w*self.fy/gamma_m1
		self.add_rslt('mdrd', round(term*1e-6, 1), 'M.D.Rd', 'kNm', '4.5.2.2', 9)
		return term

	# 
	def phi_k(self):
		a = 0.21
		term = 0.5*(1+a*(self.lambda_d_()-0.2)+self.lambda_d_()**2)
		self.add_rslt('phi_k_y', round(term, 2), '\u03A6.K,y', '4.5.1.4')
		return term

	def chi_k(self):
		term = 1/(self.phi_k()+sqrt(self.phi_k()**2-self.lambda_d_()**2))
		if term >= 1.0:
			term = 1.0
		self.add_rslt('chi_k_y', round(term, 4), '\u03C8.K,y', '4.5.1.4', 8)
		return term


class NkRd(Profile, Results):
	""" SIA 263:13, 4.5.1 Knicken """
	
	def __init__(self, entries):
		Profile.__init__(self, entries)
		if 'lk' in entries:
			self.lk = entries['lk']
		else:
			self.lk = entries['ld']
		self.production = entries['production']
		
	def alpha(self, axis):
		a, b, c, d = 0.21, 0.34, 0.49, 0.76
		shapes = ['IPE', 'PEA', 'INP', 'HEA', 'HEB', 'HEM']

		if self.production == 'rolled':
			if self.shape in shapes and axis == 'y':
				if (self.h-self.tf)/self.b > 1.2 and self.tf <= 40:
					term = a
				elif self.tf <= 100:
					term = b
				else:
					term = d
				self.add_rslt('alpha_y', term, '\u03B1.y', '-', '4.5.1.4')

			elif self.shape in shapes and axis == 'z':
				if (self.h-self.tf)/self.b > 1.2 and self.tf <= 40:
					term = b
				elif self.tf <= 100:
					term = c
				else:
					term = d
				self.add_rslt('alpha_z', term, '\u03B1.z', '-', '4.5.1.4')

			elif self.shape in ['RRW']:
				term = a
				self.add_rslt('alpha', term, '\u03B1', '-', '4.5.1.4')
			else:
				pass

		else:
			pass
		self.add_rslt('alpha', term, '\u03B1', '-', '4.5.1.4')
		return term
				
	def lambda_k_y(self):
		term = self.lk/self.iy_/(pi*sqrt(e/self.fy))
		self.add_rslt('lambda_k_y', round(term, 1), '\u03BB.K,y', '4.5.1.4')
		return term

	def lambda_k_z(self):
		term = self.lk/self.iz_/(pi*sqrt(e/self.fy))
		self.add_rslt('lambda_k_z', round(term, 1), '\u03BB.K,z', '4.5.1.4')
		return term

	def phi_k_y(self):
		term = 0.5*(1+self.alpha('y')*(self.lambda_k_y()-0.2)+self.lambda_k_y()**2)
		self.add_rslt('phi_k_y', round(term, 2), '\u03A6.K,y', '4.5.1.4')
		return term

	def phi_k_z(self):
		term = 0.5*(1+self.alpha('z')*(self.lambda_k_z()-0.2)+self.lambda_k_z()**2)
		self.add_rslt('phi_k_z', round(term, 2), '\u03A6.K,z', '4.5.1.4')
		return term

	def chi_k_y(self):
		term = 1/(self.phi_k_y()+sqrt(self.phi_k_y()**2-self.lambda_k_y()**2))
		if term >= 1.0:
			term = 1.0
		self.add_rslt('chi_k_y', round(term, 4), '\u03C8.K,y', '4.5.1.4', 8)
		return term

	def chi_k_z(self):
		term = 1/(self.phi_k_z()+sqrt(self.phi_k_z()**2-self.lambda_k_z()**2))
		if term >= 1.0:
			term = 1.0
		self.add_rslt('chi_k_z', round(term, 4), '\u03C8.K,z', '4.5.1.4', 8)
		return term
	
	def nkrd_y(self):
		term = self.chi_k_y()*self.fy*self.a/gamma_m1
		self.add_rslt('nkrd_y', round(term*1e-3, 1), 'N.K,y,Rd', 'kN', '4.5.1.3', 7)
		return term
	
	def nkrd_z(self):
		term = self.chi_k_z()*self.fy*self.a/gamma_m1
		self.add_rslt('nkrd_z', round(term*1e-3, 1), 'N.K,z,Rd', 'kN', '4.5.1.3', 7)
		return term






from math import sin, cos, radians
from .bending_force import BendingForce
from ..cat262 import Beton, Betonstahl, Cat262
from ..params import dm_set, epsilon_c2d, epsilon_ud
from ...funcs import as_
from ...results import Results


class ShearForceWithoutReinfrocement(Results):
	code = 'SIA 262:13'
	selection = ['vrd']

	def __init__(self, entries):

		self.mrd = BendingForce(entries).mrd()
		self.concrete = Beton(entries['beton'], entries['gamma_c'])
		self.betonstahl = Betonstahl(entries['betonstahl'], entries['gamma_s'])
		self.cat262 = Cat262()

		dm_inf = dm_set[entries['dm_inf']]
		s_inf = entries['s_inf']
		self.as_inf = as_(dm_inf, s_inf)
		self.d_inf = int(entries['d_inf'])

		self.w = int(entries['w'])
		self.h = int(entries['h'])

		self.dm_a = entries['dm_a']
		self.md = entries['md']*1e6

		self.concrete_type = 'normal'
		self.state = entries['state']
		self.dmax = entries['dmax']
		self.ff43323 = entries['ff43323']
		self.ff43324 = entries['ff43324']
		self.theta = entries['theta']

	def compute(self):
		self.vrd()
		return  self.get_results()

	def vrd(self):
		if self.dm_a > self.d_inf/6:
			dv = self.d_inf-self.dm_a
		else:
			dv = self.d_inf
		term = self.cat262.vrd(self.kd(), self.concrete.tau_cd(), dv)
		return term

	def kd(self):
		if self.state == 'elastic':
			if self.md == 0:
				md, mrd = 1, 1
			else:
				md, mrd = self.md, self.mrd
			epsilon_v = self.betonstahl.epsilon_v_el(md, mrd)
		else:
			epsilon_v = self.betonstahl.epsilon_v_pl()
		if self.ff43323:
			epsilon_v *= 1.5
		if self.ff43324:
			theta = radians(self.theta)
			epsilon_v *= 1/(sin(theta)**4+cos(theta)**4)
		term = self.cat262.kd(epsilon_v, self.d_inf, self.kg())
		return term

	def kg(self):
		if self.concrete_type != 'normal' or self.concrete.fck > 70:
			self.dmax = 0
		term = self.cat262.kg(self.dmax)
		return term






 
import sqlite3 as lite
import os

dir_path = os.path.dirname(os.path.abspath(__file__))
db_file = os.path.join(dir_path, 'szs.db')

class Database:
	def __init__(self):
		self.con = lite.connect(db_file)


	def select_tables(self):
		with self.con:
			cur = self.con.cursor()
			cur.execute("SELECT name FROM sqlite_master WHERE type='table';")
			data = [x[0] for x in cur.fetchall()]
			return data

	def select_table(self, table):
		with self.con:
			cur = self.con.cursor()
			cur.execute('SELECT * FROM '+table+'')
			data = [x for x in cur.fetchall()]
			return data

	def select_header(self, table):
		with self.con:
			cur = self.con.cursor()
			cur.execute('PRAGMA table_info('+table+')')
			data = [x[1] for x in cur.fetchall()]
			return data

	def select_row(self, table, row):
		with self.con:
			cur = self.con.cursor()
			cur.execute('SELECT * FROM '+table+' WHERE size=?', (row,))
			data = cur.fetchone()
			return data

	def select_col(self, table, col):
		with self.con:
			cur = self.con.cursor()
			cur.execute('SELECT '+col+' FROM '+table+'')
			data = [x[0] for x in cur.fetchall()]
			return data

	def select_row_dict(self, table, row):
		header = self.select_header(table)
		row = self.select_row(table, row)
		return dict(zip(header, row))

	def select_choices(self):
		tables = self.select_tables()
		choices = {}
		for table in tables:
			choices[table] = self.select_col(table, 'size')
		return choices
			



if __name__ == '__main__':
	cat = cat263()
	print('tables: {}'.format(Database().select_tables()))
	print('choices: {}'.format(Database().select_choices()))
	print('column from IPE: {}'.format(Database().select_col('IPE', 'size')))
	print(cat.mfd())
	print(cat.mvrd())










import unittest

from civeng.concrete.modules.bending_force import BendingForce, BendingCompression


class BendingForceTest(unittest.TestCase):
	
	def setUp(self):
		self.entries = {
			'beton': 3,
			'gamma_c': 1.5,
			'eta_t': 1.0,
			'betonstahl': 0,
			'gamma_s': 1.15,
			'as_inf': {'dm': 10, 's': 150, 'd':210},
 			'w': 1000,
			'h': 250,
			'd_inf': 260,
			}
		print('setUp executed!')
	
	def testCalculation(self):
		
		biegung = BendingForce(self.entries)
		#print(biegung.compute())

class BendingCompressionTest(unittest.TestCase):
	
	def setUp(self):
		self.entries = {
			'beton': 3,
			'gamma_c': 1.5,
			'eta_t': 1.0,
			'betonstahl': 0,
			'gamma_s': 1.15,
			'as_sup': {'dm': 3, 's': 333, 'd':40},
			'as_inf': {'dm': 4, 's': 200, 'd':410},
			'w': 300,
			'h': 450,
			}
		print('setUp executed!')

	def testCalculation(self):
		b = BendingCompression(self.entries)
		print('{}'.format(b.compute()))



if __name__ == "__main__":
	suite = unittest.TestSuite()
	suite.addTest(BendingCompressionTest('testCalculation'))
	unittest.TextTestRunner().run(suite)
	






from math import fabs, sqrt
from ..cat262 import Beton, Betonstahl
from ..params import dm_set, epsilon_c2d, epsilon_ud, es
from ...funcs import as_
from ...results import Results




class BendingForceMain:
	def __init__(self, entries):
		self.entries = entries
		self.b_as_sup = entries['b_as_sup']
		self.b_as_sup1 = entries['b_as_sup1']

	def compute(self):
		if self.b_as_sup == False and self.b_as_sup1 == False:
			return BendingForce(self.entries).compute()
		elif self.b_as_sup == True or self.b_as_sup1 == False:
			return BendingCompression(self.entries).compute()
			


class BendingForce(Results):
	code = ''
	selection = ['mrd','mrd_max_stat_d', 'as_min', 'epsilon_s_inf', 'xd']

	def __init__(self, entries):

		self.concrete = Beton(entries['beton'], entries['gamma_c'])
		self.betonstahl = Betonstahl(entries['betonstahl'], entries['gamma_s'])
		self.fcd = self.concrete.fcd()
		self.fsd = self.betonstahl.fsd()

		#dm_inf = dm_set[entries['as_inf']['dm']]
		#s_inf = entries['as_inf']['s']
		self.as_inf = entries['as_inf']
		self.d_inf = entries['d_inf']

		self.w = int(entries['w'])
		self.h = int(entries['h'])
		self.fctd = self.concrete.fctd(self.w, self.h)

	def compute(self):
		if self.w > 0 and self.h > 0 and self.d_inf > 0:
			x = self.x()
			self.mrd()
			self.epsilon_s_inf(x)
			self.xd(x)
			self.zd(x)
			self.mrd_()
			self.as_min()
			self.mrd_max_stat_d()
			self.mrd_max_stat_ind()
			self.as_max_stat_d()
			self.as_max_stat_ind()

		return  self.get_results()

	def epsilon_s_inf(self, x):
		term = epsilon_c2d/x*(self.d_inf-x)
		epsilon_yd = self.betonstahl.epsilon_yd()
		self.add_rslt2(round(term*1e3, 2))
		return term

	def x(self):
		term = self.as_inf*self.fsd/(0.85*self.w*self.fcd)
		self.add_rslt('x', round(term), 'x', 'mm')
		return term

	def z(self, x):
		term = self.d_inf-0.425*x
		self.add_rslt('z', round(term), 'z', 'mm')
		return term

	def mrd(self):
		term = self.as_inf*self.fsd*self.z(self.x())
		check = round(term/self.mrd_max_stat_d(), 1)
		self.add_rslt('mrd', round(term*1e-6, 1), 'M.Rd', 'kNm')
		return term

	def xd(self, x):
		term = x/self.d_inf
		self.add_rslt('xd', round(term, 2), 'x/d', '-')
		return term

	def zd(self, x):
		term = self.z(x)/self.d_inf
		self.add_rslt('zd', round(term, 2), 'z/d', '-')
		return term

	def mrd_(self):
		term = self.fctd*self.w*self.h**2/6
		self.add_rslt('mrd_', round(term*1e-6, 1), 'M.rd', 'kNm')
		return term

	def as_min(self):
		term = self.w*self.fcd/self.fsd*(self.d_inf-sqrt(self.d_inf**2-self.fctd*self.h**2/(3*self.fcd)))
		check = round(term/self.as_inf, 1)
		self.add_rslt('as_min', round(term, 1), 'A.s,min', 'mm^2')
		return term

	def mrd_max_stat_d(self):
		xd = 0.35
		term = 0.85*xd*self.w*self.fcd*self.d_inf**2*(1-0.425*xd)
		self.add_rslt('mrd_max_stat_d', round(term*1e-6, 1), 'M.Rd,max,0.35', 'kNm')
		return term

	def mrd_max_stat_ind(self):
		xd = 0.5
		term = 0.85*xd*self.w*self.fcd*self.d_inf**2*(1-0.425*xd)
		self.add_rslt('mrd_max_stat_ind', round(term*1e-6, 1), 'M.Rd,max,0.5', 'kNm')
		return term

	def as_max_stat_d(self):
		xd = 0.35
		term = 0.85*xd*self.w*self.fcd*self.d_inf/self.fsd
		self.add_rslt('as_max_stat_d', round(term), 'A.s,max,0.35', 'mm^2')
		return term

	def as_max_stat_ind(self):
		xd = 0.5
		term = 0.85*xd*self.w*self.fcd*self.d_inf/self.fsd
		self.add_rslt('as_max_stat_ind', round(term), 'A.s,max,0.5', 'mm^2')
		return term

from sympy.solvers import solve
from sympy import Symbol
from sympy.mpmath import findroot

class BendingCompression(Results):
	code = ''
	selection = ['x', 'mrd', 'epsilon_s_sup', 'epsilon_s_inf', 'xd', \
			  'zsd', 'dcd', 'dsd', 'sum']
	
	def __init__(self, entries):

		self.concrete = Beton(entries['beton'], entries['gamma_c'])
		self.betonstahl = Betonstahl(entries['betonstahl'], entries['gamma_s'])
		self.fcd = self.concrete.fcd()
		self.fsd = self.betonstahl.fsd()

		#dm_sup = dm_set[entries['as_sup']['dm']]
		#s_sup = entries['as_sup']['s']
		self.as_sup = entries['as_sup']
		self.d_sup= entries['d_sup']
		self.as_sup2 = entries['as_sup1']
		self.d_sup2 = entries['d_sup1']

		#dm_inf = dm_set[entries['as_inf']['dm']]
		#s_inf = entries['as_inf']['s']
		self.as_inf = entries['as_inf']
		self.d_inf = entries['d_inf']

		self.w = int(entries['w'])
		self.h = int(entries['h'])
		self.fctd = self.concrete.fctd(self.w, self.h)
		
		self.epsilon_yd = self.betonstahl.epsilon_yd()
		
	def compute(self):
		
		x = self.x_case()
		self.add_rslt('x', round(x[0]), 'x', 'mm')
		self.add_rslt('epsilon_s_sup', round(self.epsilon_s_sup(*x)*1e3, 3), 'essup', '-')
		self.add_rslt('epsilon_s_inf', round(self.epsilon_s_inf(x[0])*1e3, 3), 'esinf', '-')
		self.add_rslt('mrd', round(self.mrd_case(*x)*1e-6, 1), 'M.Rd', 'kNm')
		
		xd = round(x[0]/self.d_inf, 2)
		self.add_rslt('xd', xd, 'x/d', '-')
		if xd <= 0:
			self.add_msg('xd', 'xd <= 0')
		self.add_rslt('zsd', round(self.zsd()*1e-3, 1), 'Z.sd', 'kN')
		self.add_rslt('dcd', round(self.dcd(x[0])*1e-3, 1), 'D.cd', 'kN')
		self.add_rslt('dsd', round(self.dsd_case(*x)*1e-3, 1), 'D.sd', 'kN')
		check = self.zsd()+self.dcd(x[0])+self.dsd_case(*x)
		self.add_rslt('sum', round(check*1e-3, 1), '\u2211 = 0', 'kN')
		
		
		return self.get_results()
		

	def epsilon_s_inf(self, x):
		term = epsilon_c2d/x*(self.d_inf-x)
		return term

	def zsd(self):
		term = self.as_inf*self.fsd
		return term

	def dcd(self, x):
		term = -0.85*x*self.w*self.fcd
		return term

	# ds1d

	def epsilon_s_sup(self, x, flows):
		if flows:
			term = self.epsilon_yd
		else:
			term = epsilon_c2d/x*(x-self.d_sup)
		return term

	def sigma_s_sup(self, x, flows):
		term = self.epsilon_s_sup(x, flows)*es
		return term
	
	def dsd_case(self, x, flows):
		term = -self.as_sup*self.sigma_s_sup(x, flows)
		return term

	# end ds1d
	
	# ds2d
	
	def epsilon_s_sup2(self, x, flows):
		if flows:
			term = self.epsilon_yd
		else:
			term = epsilon_c2d/x*(x-self.d_sup2)
		return term

	def sigma_s_sup2(self, x, flows):
		term = self.epsilon_s_sup2(x, flows)*es
		return term

	def ds2d_case(self, x, flows):
		term = self.as_sup2*self.sigma_s_sup2(x, flows)
		return term 

	# end ds2d

	def x_case(self):

		flows = False

		while True:
			f = lambda x: self.zsd()+self.dsd_case(x, flows)+self.dcd(x)
			term = findroot(f, self.d_inf/2)

			if fabs(self.epsilon_s_sup(term, flows)) > self.epsilon_yd:
				flows = True
			else:
				return term, flows
				break

	def mrd_case(self, x, flows):
		term = self.dsd_case(x, flows)*(self.d_sup-self.h/2)+ \
			   self.dcd(x)*(0.425*x-self.h/2)+ \
		       self.zsd()*(self.d_inf-self.h/2)
		return term

	# end case







from setuptools import setup

setup(name='civeng',
	version='0.2.8',
	description='civil engineering tools',
	url='https://pypi.python.org/pypi/civeng',
	author='dutu8ure',
	author_email='dutu8ure@gmail.com',
	license='GPLv3',
	packages=['civeng', 
				'civeng/concrete', 
				'civeng/concrete/modules',
				'civeng/steel',
				'civeng/steel/modules',],
	package_data={'civeng/steel': ['szs.db'],},
	zip_safe=False)
